--=======================================================================
--! @tri_state_ctrl.vhd
--=======================================================================
-------------------------------------------------------------------------
--! @brief
--! Control the activation of the tri state data line in the SPI 
-------------------------------------------------------------------------
--! @details
--!
-------------------------------------------------------------------------
--! @version
--! 0.1 | mc | 07.10.2020
--!
--! @author
--! mc : Juan Fernandez, Seven Solutions SL
-------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-------------------------------------------------------------------------
-- ENTITY
-------------------------------------------------------------------------
entity ent_tri_state_ctrl is
   port (
    --------------------------------------------------------------------
    -- Clocking ports
    --------------------------------------------------------------------
    clk_i : in  std_logic;
    --------------------------------------------------------------------
    -- Data ports
    --------------------------------------------------------------------
    cs_i      : in  std_logic;
    sdata_i   : in  std_logic;
    sdata_t_o : out std_logic
  );
  attribute keep_hierarchy : string;
  attribute keep_hierarchy of ent_tri_state_ctrl : entity is "true";
end ent_tri_state_ctrl;

------------------------------------------------------------------------
-- ARCHITECTURE
------------------------------------------------------------------------
architecture arch_tri_state_ctrl of ent_tri_state_ctrl is

------------------------------------------------------------------------
-- 1. COMPONENTS DECLARATION
------------------------------------------------------------------------
------------------------------------------------------------------------
-- 2. CONSTANTS
------------------------------------------------------------------------
------------------------------------------------------------------------
-- 3. TYPES
------------------------------------------------------------------------
 

------------------------------------------------------------------------
-- 4. SIGNAL DECLARATION
------------------------------------------------------------------------
  signal s_count_cycles: integer := 0;
  signal s_sdata_t     : std_logic := '0';
  signal s_is_a_read   : std_logic := '0';
------------------------------------------------------------------------
-- 5. ATTRIBUTES
------------------------------------------------------------------------
 
------------------------------------------------------------------------
-- BEGIN
------------------------------------------------------------------------
begin

------------------------------------------------------------------------
-- 1. COMPONENT INSTANTIATION
------------------------------------------------------------------------
------------------------------------------------------------------------
-- 2. LOGIC
------------------------------------------------------------------------
sdata_t_o <= s_sdata_t;
------------------------------------------------------------------------
-- 3. PROCESS
------------------------------------------------------------------------
------------------------------------------------------------------------
-- 
------------------------------------------------------------------------
p_count : process(clk_i, cs_i)
begin
  if cs_i = '1' then
    s_count_cycles <=  0;
    s_is_a_read    <= '0';
  else
    if rising_edge(clk_i) then
      s_count_cycles <= s_count_cycles + 1;
      -- at the first rising edge we check if it is a read operation
      if s_count_cycles = 0 then
        if sdata_i = '1' then
          s_is_a_read <= '1';
        else
          s_is_a_read <= '0';
        end if;
      end if;
    end if;
  end if;
end process;

------------------------------------------------------------------------
-- 
------------------------------------------------------------------------
p_tri_state : process(clk_i, cs_i)
begin
  if cs_i = '1' then
    s_sdata_t      <= '0';
  else
    if falling_edge(clk_i) then
      -- after 16 cycles if it was a read we rise the t signal to change the tristate buffer and
      -- allow the input of data
      if s_count_cycles = 16 and s_is_a_read = '1' then
        s_sdata_t <= '1';
      end if;
    end if;
  end if;
end process;

------------------------------------------------------------------------
-- END ARCHITECTURE
------------------------------------------------------------------------
end;