//------------------------------------------------------------------------------
//  (c) Copyright 2013-2015 Xilinx, Inc. All rights reserved.
//
//  This file contains confidential and proprietary information
//  of Xilinx, Inc. and is protected under U.S. and
//  international copyright and other intellectual property
//  laws.
//
//  DISCLAIMER
//  This disclaimer is not a license and does not grant any
//  rights to the materials distributed herewith. Except as
//  otherwise provided in a valid license issued to you by
//  Xilinx, and to the maximum extent permitted by applicable
//  law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
//  WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
//  AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
//  BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
//  INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
//  (2) Xilinx shall not be liable (whether in contract or tort,
//  including negligence, or under any other theory of
//  liability) for any loss or damage of any kind or nature
//  related to, arising under or in connection with these
//  materials, including for any direct, or any indirect,
//  special, incidental, or consequential loss or damage
//  (including loss of data, profits, goodwill, or any type of
//  loss or damage suffered as a result of any action brought
//  by a third party) even if such damage or loss was
//  reasonably foreseeable or Xilinx had been advised of the
//  possibility of the same.
//
//  CRITICAL APPLICATIONS
//  Xilinx products are not designed or intended to be fail-
//  safe, or for use in any application requiring fail-safe
//  performance, such as life-support or safety devices or
//  systems, Class III medical devices, nuclear facilities,
//  applications related to the deployment of airbags, or any
//  other applications that could lead to death, personal
//  injury, or severe property or environmental damage
//  (individually and collectively, "Critical
//  Applications"). Customer assumes the sole risk and
//  liability of any use of Xilinx products in Critical
//  Applications, subject only to applicable laws and
//  regulations governing limitations on product liability.
//
//  THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
//  PART OF THIS FILE AT ALL TIMES.
//------------------------------------------------------------------------------


`timescale 1ps/1ps

// =====================================================================================================================
// This example design top module instantiates the example design wrapper; slices vectored ports for per-channel
// assignment; and instantiates example resources such as buffers, pattern generators, and pattern checkers for core
// demonstration purposes
// =====================================================================================================================

module wr_gtph4_wrapper_example_top (

  // Differential reference clock inputs
  input  wire mgtrefclk0_x1y2_p,
  input  wire mgtrefclk0_x1y2_n,

  // Serial data ports for transceiver channel 0
  input  wire ch0_gthrxn_in,
  input  wire ch0_gthrxp_in,
  output wire ch0_gthtxn_out,
  output wire ch0_gthtxp_out,

  // User-provided ports for reset helper block(s)
  input  wire hb_gtwiz_reset_clk_freerun_in,
  input  wire hb_gtwiz_reset_all_in,


  //Manually added.
  output wire        tx_clk_o,
  input wire [1:0] 	 tx_k_i,
  input wire [15:0]  tx_data_i,

  output wire        rx_clk_o,
  output wire [1:0]  rx_k_o,
  output wire [15:0] rx_data_o,

  input wire        rx_slide_i,
  output wire 	    rx_byte_is_aligned_o,
  output wire       rx_comma_det_o,

  input wire        rx_cdr_reset_i,

  output wire       ready_o);

 // ===================================================================================================================
 //Manual assignation
 // ===================================================================================================================

 assign tx_clk_o = gtwiz_userclk_tx_usrclk2_int;
 assign rx_clk_o = gtwiz_userclk_rx_usrclk2_int;

 //--------------------------------------------------------------------------------------------------------------------
 wire [15:0] txctrl0_int;
 wire [15:0] txctrl1_int;
 wire [7:0]  txctrl2_int;


 assign txctrl0_int = 16'b0;
 assign txctrl1_int = 16'b0;
 assign txctrl2_int = {6'b0, tx_k_i};
 assign rx_k_o = rxctrl0_int[1:0];

 //--------------------------------------------------------------------------------------------------------------------

  // ===================================================================================================================
  // PER-CHANNEL SIGNAL ASSIGNMENTS
  // ===================================================================================================================

  // The core and example design wrapper vectorize ports across all enabled transceiver channel and common instances for
  // simplicity and compactness. This example design top module assigns slices of each vector to individual, per-channel
  // signal vectors for use if desired. Signals which connect to helper blocks are prefixed "hb#", signals which connect
  // to transceiver common primitives are prefixed "cm#", and signals which connect to transceiver channel primitives
  // are prefixed "ch#", where "#" is the sequential resource number.
   wire [0:0] gtwiz_buffbypass_tx_done_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_buffbypass_tx_error_int;
 //--------------------------------------------------------------------------------------------------------------------
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_buffbypass_tx_reset_int;
 //--------------------------------------------------------------------------------------------------------------------
   wire [0:0] cplllock_int;
 //--------------------------------------------------------------------------------------------------------------------
   wire [0:0] cpllreset_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_userclk_tx_reset_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_userclk_tx_srcclk_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_userclk_tx_usrclk_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_userclk_tx_usrclk2_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_userclk_tx_active_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_userclk_rx_reset_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_userclk_rx_srcclk_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_userclk_rx_usrclk_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_userclk_rx_usrclk2_int;
  //--------------------------------------------------------------------------------------------------------------------
   wire [0:0] gtwiz_userclk_rx_active_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_reset_tx_pll_and_datapath_int = 1'b0;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_reset_tx_datapath_int = 1'b0;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] gtwiz_reset_rx_pll_and_datapath_int = 1'b0;
  //--------------------------------------------------------------------------------------------------------------------
   wire [0:0] gtwiz_reset_rx_cdr_stable_int;
  //--------------------------------------------------------------------------------------------------------------------
   wire [0:0] gtwiz_reset_tx_done_int;
  //--------------------------------------------------------------------------------------------------------------------
   wire [0:0] gtwiz_reset_rx_done_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] rx8b10ben_int = 1'b1;
  //--------------------------------------------------------------------------------------------------------------------
   wire [0:0] rxbufreset_int = 1'b0;
  //--------------------------------------------------------------------------------------------------------------------
   wire [0:0] rxcdrreset_int;
  //--------------------------------------------------------------------------------------------------------------------
   wire [0:0] rxcommadeten_int = 1'b1;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] rxmcommaalignen_int = 1'b0;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] rxpcommaalignen_int = 1'b0;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] tx8b10ben_int = 1'b1;
  //--------------------------------------------------------------------------------------------------------------------
   wire [0:0] gtpowergood_int;
  //--------------------------------------------------------------------------------------------------------------------
   wire [0:0] rxbyterealign_int;
  //--------------------------------------------------------------------------------------------------------------------
   wire [1:0] rxclkcorcnt_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [15:0] rxctrl0_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [15:0] rxctrl1_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [7:0] rxctrl2_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [7:0] rxctrl3_int;
  //--------------------------------------------------------------------------------------------------------------------
   wire [0:0] rxpmaresetdone_int;
  //--------------------------------------------------------------------------------------------------------------------
  wire [0:0] txpmaresetdone_int;

  // ===================================================================================================================
  // BUFFERS
  // ===================================================================================================================

  // For GTH core configurations which utilize the transceiver channel CPLL, the drpclk_in port must be driven by
  // the free-running clock at the exact frequency specified during core customization, for reliable bring-up
  // Globally buffer the free-running input clock
 wire hb_gtwiz_reset_clk_freerun_buf_int;

  BUFG bufg_clk_freerun_inst (
    .I (hb_gtwiz_reset_clk_freerun_in),
    .O (hb_gtwiz_reset_clk_freerun_buf_int)
  );

  // Instantiate a differential reference clock buffer for each reference clock differential pair in this configuration,
  // and assign the single-ended output of each differential reference clock buffer to the appropriate PLL input signal

  // Differential reference clock buffer for MGTREFCLK0_X1Y2
  wire mgtrefclk0_x1y2_int;

  IBUFDS_GTE4 #(
    .REFCLK_EN_TX_PATH  (1'b0),
    .REFCLK_HROW_CK_SEL (2'b00),
    .REFCLK_ICNTL_RX    (2'b00)
  ) IBUFDS_GTE4_MGTREFCLK0_X1Y2_INST (
    .I     (mgtrefclk0_x1y2_p),
    .IB    (mgtrefclk0_x1y2_n),
    .CEB   (1'b0),
    .O     (mgtrefclk0_x1y2_int),
    .ODIV2 ()
  );

  // ===================================================================================================================
  // USER CLOCKING RESETS
  // ===================================================================================================================

  // The TX user clocking helper block should be held in reset until the clock source of that block is known to be
  // stable. The following assignment is an example of how that stability can be determined, based on the selected TX
  // user clock source. Replace the assignment with the appropriate signal or logic to achieve that behavior as needed.
  assign gtwiz_userclk_tx_reset_int = ~(&txpmaresetdone_int);

  // The RX user clocking helper block should be held in reset until the clock source of that block is known to be
  // stable. The following assignment is an example of how that stability can be determined, based on the selected RX
  // user clock source. Replace the assignment with the appropriate signal or logic to achieve that behavior as needed.
  assign gtwiz_userclk_rx_reset_int = ~(&rxpmaresetdone_int);

  // The TX buffer bypass controller helper block should be held in reset until the TX user clocking network helper
  // block which drives it is active
  (* DONT_TOUCH = "TRUE" *)
  wr_gth_wrapper_example_reset_synchronizer reset_synchronizer_gtwiz_buffbypass_tx_reset_inst (
    .clk_in  (gtwiz_userclk_tx_usrclk2_int),
    .rst_in  (~gtwiz_userclk_tx_active_int),
    .rst_out (gtwiz_buffbypass_tx_reset_int)
  );

  // ===================================================================================================================
  // INITIALIZATION
  // ===================================================================================================================

  // Buffer the hb_gtwiz_reset_all_in input and logically combine it with the internal signal from the example
  // initialization block as well as the VIO-sourced reset
   wire hb_gtwiz_reset_all_buf_int;
   wire hb_gtwiz_reset_all_init_int;
   wire hb_gtwiz_reset_all_int;

  IBUF ibuf_hb_gtwiz_reset_all_inst (
    .I (hb_gtwiz_reset_all_in),
    .O (hb_gtwiz_reset_all_buf_int)
  );

  // Declare the receiver reset signals that interface to the reset controller helper block. For this configuration,
  // which uses the same PLL type for transmitter and receiver, the "reset RX PLL and datapath" feature is not used.
  wire hb_gtwiz_reset_rx_pll_and_datapath_int = 1'b0;

  // Declare signals which connect the VIO instance to the initialization module for debug purposes
   wire       init_done_int;
   wire [3:0] init_retry_ctr_int;

  // Combine the receiver reset signals form the initialization module and the VIO to drive the appropriate reset
  // controller helper block reset input
  wire hb_gtwiz_reset_rx_datapath_init_int;

  // The example initialization module interacts with the reset controller helper block and other example design logic
  // to retry failed reset attempts in order to mitigate bring-up issues such as initially-unavilable reference clocks
  // or data connections. It also resets the receiver in the event of link loss in an attempt to regain link, so please
  // note the possibility that this behavior can have the effect of overriding or disturbing user-provided inputs that
  // destabilize the data stream. It is a demonstration only and can be modified to suit your system needs.
  wr_gtph4_wrapper_example_init example_init_inst (
    .clk_freerun_in  (hb_gtwiz_reset_clk_freerun_buf_int),
    .reset_all_in    (hb_gtwiz_reset_all_buf_int),  //button
    .tx_init_done_in (gtwiz_reset_tx_done_int && gtwiz_buffbypass_tx_done_int),
    .rx_init_done_in (gtwiz_reset_rx_done_int && gtwiz_reset_rx_cdr_stable_int),
    .rx_data_good_in (1'b1),
    .reset_all_out   (hb_gtwiz_reset_all_init_int),
    .reset_rx_out    (hb_gtwiz_reset_rx_datapath_init_int),
    .init_done_out   (init_done_int),
    .retry_ctr_out   (init_retry_ctr_int),

    .cplllock_in     (cplllock_int),
    .cpllreset_out   (cpllreset_int)
  );

  assign ready_o = ~(hb_gtwiz_reset_all_int || ~gtwiz_reset_rx_done_int || ~gtwiz_buffbypass_tx_done_int || ~cplllock_int || ~gtwiz_reset_rx_cdr_stable_int || ~init_done_int);

  // button + init_module
  // hb_gtwiz_reset_all_buf_int || //Just the init module now.
  assign hb_gtwiz_reset_all_int =  hb_gtwiz_reset_all_init_int;

  // ===================================================================================================================
  // EXAMPLE WRAPPER INSTANCE
  // ===================================================================================================================

  // Instantiate the example design wrapper, mapping its enabled ports to per-channel internal signals and example
  // resources as appropriate
  wr_gth4_wrapper example_wrapper_inst (
    .gthrxn_in                               (ch0_gthrxn_in)
   ,.gthrxp_in                               (ch0_gthrxp_in)
   ,.gthtxn_out                              (ch0_gthtxn_out)
   ,.gthtxp_out                              (ch0_gthtxp_out)
   ,.gtwiz_userclk_tx_reset_in               (gtwiz_userclk_tx_reset_int)
   ,.gtwiz_userclk_tx_srcclk_out             (gtwiz_userclk_tx_srcclk_int)
   ,.gtwiz_userclk_tx_usrclk_out             (gtwiz_userclk_tx_usrclk_int)
   ,.gtwiz_userclk_tx_usrclk2_out            (gtwiz_userclk_tx_usrclk2_int)
   ,.gtwiz_userclk_tx_active_out             (gtwiz_userclk_tx_active_int)
   ,.gtwiz_userclk_rx_reset_in               (gtwiz_userclk_rx_reset_int)
   ,.gtwiz_userclk_rx_srcclk_out             (gtwiz_userclk_rx_srcclk_int)
   ,.gtwiz_userclk_rx_usrclk_out             (gtwiz_userclk_rx_usrclk_int)
   ,.gtwiz_userclk_rx_usrclk2_out            (gtwiz_userclk_rx_usrclk2_int)
   ,.gtwiz_userclk_rx_active_out             (gtwiz_userclk_rx_active_int)
   ,.gtwiz_reset_clk_freerun_in              ({1{hb_gtwiz_reset_clk_freerun_buf_int}})
   ,.gtwiz_reset_all_in                      ({1{hb_gtwiz_reset_all_int}})
   ,.gtwiz_reset_tx_pll_and_datapath_in      (gtwiz_reset_tx_pll_and_datapath_int)
   ,.gtwiz_reset_tx_datapath_in              (gtwiz_reset_tx_datapath_int)
   ,.gtwiz_reset_rx_pll_and_datapath_in      ({1{hb_gtwiz_reset_rx_pll_and_datapath_int}})
   ,.gtwiz_reset_rx_datapath_in              ({1{hb_gtwiz_reset_rx_datapath_init_int}})
   ,.gtwiz_reset_rx_cdr_stable_out           (gtwiz_reset_rx_cdr_stable_int)
   ,.gtwiz_reset_tx_done_out                 (gtwiz_reset_tx_done_int)
   ,.gtwiz_reset_rx_done_out                 (gtwiz_reset_rx_done_int)
   ,.gtwiz_userdata_tx_in                    (tx_data_i)
   ,.gtwiz_userdata_rx_out                   (rx_data_o)
   ,.drpclk_in                               (hb_gtwiz_reset_clk_freerun_buf_int)
   ,.gtrefclk0_in                            (mgtrefclk0_x1y2_int)
   ,.rx8b10ben_in                            (rx8b10ben_int)
   ,.rxbufreset_in                           (rxbufreset_int)
   ,.rxcdrreset_in                           (rx_cdr_reset_i)
   ,.rxcommadeten_in                         (rxcommadeten_int)
   ,.rxmcommaalignen_in                      (rxmcommaalignen_int)
   ,.rxpcommaalignen_in                      (rxpcommaalignen_int)
   ,.rxslide_in                              (rx_slide_i)
   ,.tx8b10ben_in                            (tx8b10ben_int)
   ,.txctrl0_in                              (txctrl0_int)
   ,.txctrl1_in                              (txctrl1_int)
   ,.txctrl2_in                              (txctrl2_int)
   ,.gtpowergood_out                         (gtpowergood_int)
   ,.rxbufstatus_out                         (rxbufstatus_int)
   ,.rxbyteisaligned_out                     (rx_byte_is_aligned_o)
   ,.rxbyterealign_out                       (rxbyterealign_int)
   ,.rxclkcorcnt_out                         (rxclkcorcnt_int)
   ,.rxcommadet_out                          (rx_comma_det_o)
   ,.rxctrl0_out                             (rxctrl0_int)
   ,.rxctrl1_out                             (rxctrl1_int)
   ,.rxctrl2_out                             (rxctrl2_int)
   ,.rxctrl3_out                             (rxctrl3_int)
   ,.rxpmaresetdone_out                      (rxpmaresetdone_int)
   ,.txpmaresetdone_out                      (txpmaresetdone_int)
   ,.gtwiz_buffbypass_tx_reset_in            (gtwiz_buffbypass_tx_reset_int)
   ,.gtwiz_buffbypass_tx_start_user_in       (1'b0)
   ,.gtwiz_buffbypass_tx_done_out            (gtwiz_buffbypass_tx_done_int)
   ,.gtwiz_buffbypass_tx_error_out           (gtwiz_buffbypass_tx_error_int)
   ,.cplllock_out                            (cplllock_int)
   ,.cplllocken_in                           (1'b1)
   ,.cpllreset_in                            (cpllreset_int)
);


endmodule
