#! /bin/bash

# Variables
export BUILDROOT="buildroot-2017.02"
export BASEDIR=$(pwd)

# Step 0: Clean folders
if [ -e ${BASEDIR}/download ]; then
	rm -r ${BASEDIR}/download
fi

if [ -e ${BASEDIR}/output ]; then
	rm -r ${BASEDIR}/output
fi

mkdir -p ${BASEDIR}/download
mkdir -p ${BASEDIR}/output

# Step 1: Download buildroot from Web
cd ${BASEDIR}/download
wget https://buildroot.org/downloads/${BUILDROOT}.tar.bz2
tar xvf ${BUILDROOT}.tar.bz2
mv ${BUILDROOT} ${BASEDIR}/output
cd ${BASEDIR}

# Step 1.5: Add the custom patches (required for GCC 6.X)
cp ${BASEDIR}/patches/custom/7777-gcc-usban-file-compilation-error.patch ${BASEDIR}/output/${BUILDROOT}/package/gcc/6.3.0/

# Step 2: Configure buildroot
cp ${BASEDIR}/configs/buildroot/default.config ${BASEDIR}/output/${BUILDROOT}/.config
make -C ${BASEDIR}/output/${BUILDROOT} olddefconfig

# Step 3: Compile the toolchain for aarch64
make -C ${BASEDIR}/output/${BUILDROOT} toolchain

# Step 4: Compile the u-boot
make -C ${BASEDIR}/output/${BUILDROOT} uboot

# Step 5: Compile the kernel (linux)
make -C ${BASEDIR}/output/${BUILDROOT} linux

# Step 6: Compile the devicetree and install it
export ARCH=arm64
export INSTALL_ROOTFS=${BASEDIR}/output/${BUILDROOT}/output/images/wr
export CROSS_COMPILE=${BASEDIR}/output/${BUILDROOT}/output/host/usr/bin/aarch64-buildroot-linux-gnu-
export LINUX=${BASEDIR}/output/${BUILDROOT}/output/build/linux-xilinx-v2018.3
export INSTALL_MOD_PATH=${BASEDIR}/output/${BUILDROOT}/output/images/wr
make -C ${BASEDIR}/dts
make -C ${BASEDIR}/dts install

# Step 7: Compile external drivers and install them
make -C ${BASEDIR}/drivers
make -C ${BASEDIR}/drivers install

# Step 8: Compile external tools and install them
make -C ${BASEDIR}/tools
make -C ${BASEDIR}/tools install

# Step 9: Copy external drivers and tools to skeleton
if [ -e ${BASEDIR}/output/${BUILDROOT}/output/target/wr ]; then
	rm -r ${BASEDIR}/output/${BUILDROOT}/output/target/wr
fi
cp -r ${BASEDIR}/output/${BUILDROOT}/output/images/wr ${BASEDIR}/output/${BUILDROOT}/output/target/wr

# Step 10: Generate the rootfs (skeleton)
unset ARCH
unset INSTALL_ROOTFS
unset CROSS_COMPILE
unset LINUX
unset INSTALL_MOD_PATH

make -C ${BASEDIR}/output/${BUILDROOT}
if [ ! $? -eq 0 ] ; then
	cd ${BASEDIR}/output/${BUILDROOT}
	patch -p1 < ${BASEDIR}/patches/buildroot/0001-fix-automake.patch
	cd ${BASEDIR}
	make -C ${BASEDIR}/output/${BUILDROOT}
fi

# Step 11: Clean final images 
if [ -e ${BASEDIR}/images ]; then
	rm -r ${BASEDIR}/images
fi

# Step 12: Generate final images (install mkimage if not installed, in uboot-tools pkg)
mkdir -p ${BASEDIR}/images
cp ${BASEDIR}/create_fdt_image.its ${BASEDIR}/images
cp ${BASEDIR}/binaries/BOOT.bin ${BASEDIR}/images/BOOT.bin
cp ${BASEDIR}/output/${BUILDROOT}/output/images/wr/dts/devicetree-zcu102.dtb ${BASEDIR}/images/devicetree.dtb
cp ${BASEDIR}/output/${BUILDROOT}/output/images/Image ${BASEDIR}/images
cp ${BASEDIR}/output/${BUILDROOT}/output/images/rootfs.cpio ${BASEDIR}/images/uramdisk.cpio
cd ${BASEDIR}/images
mkimage -f ${BASEDIR}/images/create_fdt_image.its ${BASEDIR}/images/image.ub
cd ${BASEDIR}

# Finally the remaining vars
unset BUILDROOT
unset BASEDIR

