#! /bin/bash

# Variables
export BUILDROOT="buildroot-2017.02"
export BASEDIR=$(pwd)

# Step 6: Compile the devicetree and install it
export ARCH=arm64
export INSTALL_ROOTFS=${BASEDIR}/output/${BUILDROOT}/output/images/wr
export CROSS_COMPILE=${BASEDIR}/output/${BUILDROOT}/output/host/usr/bin/aarch64-buildroot-linux-gnu-
export LINUX=${BASEDIR}/output/${BUILDROOT}/output/build/linux-xilinx-v2018.3
export INSTALL_MOD_PATH=${BASEDIR}/output/${BUILDROOT}/output/images/wr
make -C ${BASEDIR}/dts
make -C ${BASEDIR}/dts install

# Step 7: Compile external drivers and install them
make -C ${BASEDIR}/drivers
make -C ${BASEDIR}/drivers install

# Step 8: Compile external tools and install them
make -C ${BASEDIR}/tools
make -C ${BASEDIR}/tools install

# Step 9: Copy external drivers and tools to skeleton
if [ -e ${BASEDIR}/output/${BUILDROOT}/output/target/wr ]; then
	rm -r ${BASEDIR}/output/${BUILDROOT}/output/target/wr
fi
cp -r ${BASEDIR}/output/${BUILDROOT}/output/images/wr ${BASEDIR}/output/${BUILDROOT}/output/target/wr

# Step 10: Generate the rootfs (skeleton)
unset ARCH
unset INSTALL_ROOTFS
unset CROSS_COMPILE
unset LINUX
unset INSTALL_MOD_PATH

make -C ${BASEDIR}/output/${BUILDROOT}
if [ ! $? -eq 0 ] ; then
	cd ${BASEDIR}/output/${BUILDROOT}
	patch -p1 < ${BASEDIR}/patches/buildroot/0001-fix-automake.patch
	cd ${BASEDIR}
	make -C ${BASEDIR}/output/${BUILDROOT}
fi

# Step 11: Clean final images 
if [ -e ${BASEDIR}/images ]; then
	rm -r ${BASEDIR}/images
fi

# Step 12: Generate final images (install mkimage if not installed, in uboot-tools pkg)
mkdir -p ${BASEDIR}/images
cp ${BASEDIR}/create_fdt_image.its ${BASEDIR}/images
cp ${BASEDIR}/binaries/BOOT.bin ${BASEDIR}/images/BOOT.bin
cp ${BASEDIR}/output/${BUILDROOT}/output/images/wr/dts/devicetree-zcu102.dtb ${BASEDIR}/images/devicetree.dtb
cp ${BASEDIR}/output/${BUILDROOT}/output/images/Image ${BASEDIR}/images
cp ${BASEDIR}/output/${BUILDROOT}/output/images/rootfs.cpio ${BASEDIR}/images/uramdisk.cpio
cd ${BASEDIR}/images
mkimage -f ${BASEDIR}/images/create_fdt_image.its ${BASEDIR}/images/image.ub
cd ${BASEDIR}

# Finally the remaining vars
unset BUILDROOT
unset BASEDIR

