#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include <errno.h>

typedef struct {
    int fd;
    void *fpga_base_virt; /* Memory mapped for setting  */
    size_t size; 
    uint64_t physical_buffer_address;
} FPGABGD_DEV_STRUCT, *FPGABGD_DEV_HANDLER;

/* Open a /dev/mem and return a FPGADGV_DEV_HANDLER structure */
FPGABGD_DEV_HANDLER 
FPGABGD_open( uint64_t base, uint32_t size);

/* Close opened file descriptor */
void 
FPGABGD_close( FPGABGD_DEV_HANDLER hDev );

/* Check if hDev has been openend propertly  */
bool 
FPGABGD_isValid( FPGABGD_DEV_HANDLER hDev );

/* Set/Get funcions in the structure  */
void 
set_filedescriptor( FPGABGD_DEV_HANDLER hDev, int const _fd );
int 
get_filedescriptor( FPGABGD_DEV_HANDLER hDev );

/* Write and Read 32 bits */
uint32_t 
FPGABGD_WB_read_32( FPGABGD_DEV_HANDLER hDev, uint32_t reg );
void 
FPGABGD_WB_write_32( FPGABGD_DEV_HANDLER hDev, uint32_t reg, uint32_t val );

/* Write and Read 16 bits */
uint16_t 
FPGABGD_WB_read_16( FPGABGD_DEV_HANDLER hDev, uint32_t reg );
void 
FPGABGD_WB_write_16( FPGABGD_DEV_HANDLER hDev, uint32_t reg, uint16_t val );

/* Write and Read 8 bits */
uint8_t 
FPGABGD_WB_read_8( FPGABGD_DEV_HANDLER hDev, uint32_t reg );
void 
FPGABGD_WB_write_8( FPGABGD_DEV_HANDLER hDev, uint32_t reg, uint8_t val );

/* Write and Read 1 bit from a selected position */
uint32_t 
FPGABGD_WB_read_bit( FPGABGD_DEV_HANDLER hDev, uint32_t reg, int pos_bit );
void 
FPGABGD_WB_write_bit( FPGABGD_DEV_HANDLER hDev, uint32_t reg, int bit_pos, int bit_val );

/* Write or Read depending on fToDev value  (1 = write, 0 = read)*/
int 
FPGABGD_wishbone_RW_32( FPGABGD_DEV_HANDLER hDev, uint32_t addr, uint32_t *pData, int fToDev );

/* Returns pointer from a selected mapped place on the mapping */
uint32_t *
FPGABGD_get_buffer( FPGABGD_DEV_HANDLER hDev, uint32_t dev_addr, uint32_t nsize, bool to_dev );

/* Auxiliar function for printing a block of data*/
void
FPGABGD_print_block( uint64_t base, uint32_t offset, uint32_t size );

